"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("@nativescript/core/utils/types");
var image_source_1 = require("@nativescript/core/image-source");
var Https = require("./https.common");
var file_system_1 = require("@nativescript/core/file-system");
var peer = {
    enabled: false,
    allowInvalidCertificates: false,
    validatesDomainName: true,
};
var cache;
function setCache(options) {
    if (options) {
        cache = new okhttp3.Cache(new java.io.File(options.diskLocation), options.diskSize);
    }
    else {
        cache = null;
    }
    if (Client) {
        getClient(true);
    }
}
exports.setCache = setCache;
function clearCache() {
    if (cache) {
        cache.evictAll();
    }
}
exports.clearCache = clearCache;
var _timeout = 10;
var HttpsResponse = (function () {
    function HttpsResponse(response, tag, url) {
        this.response = response;
        this.tag = tag;
        this.url = url;
    }
    HttpsResponse.prototype.getOrCreateCloseCallback = function () {
        if (!notClosedResponses[this.tag]) {
            notClosedResponses[this.tag] = this.response;
            this.response.closeCallback = new OkHttpResponse.OkHttpResponseCloseCallback({
                onClose: function () {
                    delete notClosedResponses[this.tag];
                },
            });
        }
    };
    HttpsResponse.prototype.getCallback = function (resolve, reject) {
        return new com.nativescript.https.OkHttpResponse.OkHttpResponseAsyncCallback({
            onBitmap: function (res) {
                resolve(new image_source_1.ImageSource(res));
            },
            onString: function (res) {
                resolve(res);
            },
            onByteArray: function (res) {
                resolve(ArrayBuffer.from(res));
            },
            onFile: function (res) {
                resolve(res);
            },
            onException: function (err) {
                reject(err);
            },
        });
    };
    HttpsResponse.prototype.toArrayBuffer = function () {
        try {
            this.arrayBuffer =
                this.arrayBuffer ||
                    ArrayBuffer.from(this.response.toByteArray());
            return this.arrayBuffer;
        }
        catch (_a) {
            return null;
        }
    };
    HttpsResponse.prototype.toArrayBufferAsync = function () {
        var _this = this;
        if (this.arrayBuffer) {
            return Promise.resolve(this.arrayBuffer);
        }
        return new Promise(function (resolve, reject) {
            _this.getOrCreateCloseCallback();
            _this.response.toByteArrayAsync(_this.getCallback(resolve, reject));
        }).then(function (r) {
            _this.arrayBuffer = r;
            return _this.arrayBuffer;
        });
    };
    HttpsResponse.prototype.toString = function () {
        try {
            this.stringResponse =
                this.stringResponse || this.response.asString();
            return this.stringResponse;
        }
        catch (_a) {
            return null;
        }
    };
    HttpsResponse.prototype.toStringAsync = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (this.stringResponse) {
                            return [2, this.stringResponse];
                        }
                        _a = this;
                        return [4, new Promise(function (resolve, reject) {
                                _this.getOrCreateCloseCallback();
                                _this.response.asStringAsync(_this.getCallback(resolve, reject));
                            })];
                    case 1:
                        _a.stringResponse = _b.sent();
                        return [2, this.stringResponse];
                }
            });
        });
    };
    HttpsResponse.prototype.toJSON = function (encoding) {
        try {
            if (this.jsonResponse) {
                return this.jsonResponse;
            }
            this.stringResponse =
                this.stringResponse || this.response.asString();
            this.jsonResponse = Https.parseJSON(this.stringResponse);
            return this.jsonResponse;
        }
        catch (err) {
            console.error("HttpsResponse.toJSON", err);
            return null;
        }
    };
    HttpsResponse.prototype.toJSONAsync = function () {
        return __awaiter(this, void 0, void 0, function () {
            var r;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.jsonResponse) {
                            return [2, this.jsonResponse];
                        }
                        if (this.stringResponse) {
                            this.jsonResponse = Https.parseJSON(this.stringResponse);
                            return [2, this.jsonResponse];
                        }
                        return [4, this.toStringAsync()];
                    case 1:
                        r = _a.sent();
                        this.jsonResponse = Https.parseJSON(r);
                        return [2, this.jsonResponse];
                }
            });
        });
    };
    HttpsResponse.prototype.toImage = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                if (this.imageSource) {
                    return [2, this.imageSource];
                }
                return [2, new Promise(function (resolve, reject) {
                        _this.getOrCreateCloseCallback();
                        _this.response
                            .toBitmapAsync(_this.getCallback(resolve, reject))
                            .then(function (r) {
                            _this.imageSource = r;
                            return r;
                        });
                    })];
            });
        });
    };
    HttpsResponse.prototype.toFile = function (destinationFilePath) {
        var _this = this;
        if (this.file) {
            return Promise.resolve(this.file);
        }
        if (!destinationFilePath) {
            destinationFilePath = Https.getFilenameFromUrl(this.url);
        }
        return new Promise(function (resolve, reject) {
            _this.getOrCreateCloseCallback();
            _this.response.toFileAsync(destinationFilePath, _this.getCallback(resolve, reject));
        }).then(function () {
            _this.file = file_system_1.File.fromPath(destinationFilePath);
            return _this.file;
        });
    };
    return HttpsResponse;
}());
function enableSSLPinning(options) {
    if (!peer.host && !peer.certificate) {
        var certificate = void 0;
        var inputStream = void 0;
        try {
            var file = new java.io.File(options.certificate);
            inputStream = new java.io.FileInputStream(file);
            var x509Certificate = java.security.cert.CertificateFactory.getInstance("X509").generateCertificate(inputStream);
            peer.x509Certificate = x509Certificate;
            certificate = okhttp3.CertificatePinner.pin(x509Certificate);
            inputStream.close();
        }
        catch (error) {
            try {
                if (inputStream) {
                    inputStream.close();
                }
            }
            catch (e) { }
            console.error("nativescript-https > enableSSLPinning error", error);
            return;
        }
        peer.host = options.host;
        peer.commonName = options.commonName || options.host;
        peer.certificate = certificate;
        if (options.allowInvalidCertificates === true) {
            peer.allowInvalidCertificates = true;
        }
        if (options.validatesDomainName === false) {
            peer.validatesDomainName = false;
        }
    }
    peer.enabled = true;
    getClient(true);
}
exports.enableSSLPinning = enableSSLPinning;
function disableSSLPinning() {
    peer.enabled = false;
    getClient(true);
}
exports.disableSSLPinning = disableSSLPinning;
var Client;
var cookieJar;
var cookieManager;
function getClient(reload, timeout) {
    if (reload === void 0) { reload = false; }
    if (timeout === void 0) { timeout = 10; }
    if (Client && reload === false) {
        if (timeout === _timeout) {
            return Client;
        }
        else {
            return Client.newBuilder()
                .connectTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS)
                .writeTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS)
                .readTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS)
                .build();
        }
    }
    if (!cookieJar) {
        cookieManager = new java.net.CookieManager();
        cookieManager.setCookiePolicy(java.net.CookiePolicy.ACCEPT_ALL);
        cookieJar = new com.nativescript.https.QuotePreservingCookieJar(cookieManager);
    }
    var client = new okhttp3.OkHttpClient.Builder();
    if (peer.enabled === true) {
        if (peer.host || peer.certificate) {
            var spec = okhttp3.ConnectionSpec.MODERN_TLS;
            client.connectionSpecs(java.util.Collections.singletonList(spec));
            var pinner = new okhttp3.CertificatePinner.Builder();
            pinner.add(peer.host, [peer.certificate]);
            client.certificatePinner(pinner.build());
            if (peer.allowInvalidCertificates === false) {
                try {
                    var x509Certificate = peer.x509Certificate;
                    var keyStore = java.security.KeyStore.getInstance(java.security.KeyStore.getDefaultType());
                    keyStore.load(null, null);
                    keyStore.setCertificateEntry("CA", x509Certificate);
                    var keyManagerFactory = javax.net.ssl.KeyManagerFactory.getInstance("X509");
                    keyManagerFactory.init(keyStore, null);
                    var keyManagers = keyManagerFactory.getKeyManagers();
                    var trustManagerFactory = javax.net.ssl.TrustManagerFactory.getInstance(javax.net.ssl.TrustManagerFactory.getDefaultAlgorithm());
                    trustManagerFactory.init(keyStore);
                    var sslContext = javax.net.ssl.SSLContext.getInstance("TLS");
                    sslContext.init(keyManagers, trustManagerFactory.getTrustManagers(), new java.security.SecureRandom());
                    client.sslSocketFactory(sslContext.getSocketFactory());
                }
                catch (error) {
                    console.error("nativescript-https > client.allowInvalidCertificates error", error);
                }
            }
            if (peer.validatesDomainName === true) {
                try {
                    client.hostnameVerifier(new javax.net.ssl.HostnameVerifier({
                        verify: function (hostname, session) {
                            var pp = session.getPeerPrincipal().getName();
                            var hv = javax.net.ssl.HttpsURLConnection.getDefaultHostnameVerifier();
                            if (peer.commonName &&
                                peer.commonName[0] === "*") {
                                return (hv.verify(peer.host, session) &&
                                    hostname.indexOf(peer.host) > -1 &&
                                    hostname.indexOf(session.getPeerHost()) > -1 &&
                                    pp.indexOf(peer.commonName) !== -1);
                            }
                            else {
                                return (hv.verify(peer.host, session) &&
                                    peer.host === hostname &&
                                    peer.host === session.getPeerHost() &&
                                    pp.indexOf(peer.host) !== -1);
                            }
                        },
                    }));
                }
                catch (error) {
                    console.error("nativescript-https > client.validatesDomainName error", error);
                }
            }
        }
        else {
            console.warn("nativescript-https > Undefined host or certificate. SSL pinning NOT working!!!");
        }
    }
    _timeout = timeout;
    client
        .connectTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS)
        .writeTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS)
        .readTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS);
    if (cache) {
        client.cache(cache);
    }
    if (cookieJar) {
        client.cookieJar(cookieJar);
    }
    Client = client.build();
    return Client;
}
function cancelRequest(tag, client) {
    if (notClosedResponses[tag]) {
        notClosedResponses[tag].cancel();
        return;
    }
    var dispatcher = client.dispatcher();
    if (dispatcher.queuedCallsCount() > 0) {
        var queuedCalls = dispatcher.queuedCalls();
        for (var index_1 = 0; index_1 < queuedCalls.size(); index_1++) {
            var call = queuedCalls.get(index_1);
            if (call.request().tag() === tag) {
                call.cancel();
                return;
            }
        }
    }
    if (dispatcher.runningCallsCount() > 0) {
        var runningCalls = dispatcher.runningCalls();
        for (var index_2 = 0; index_2 < runningCalls.size(); index_2++) {
            var call = runningCalls.get(index_2);
            if (call.request().tag() === tag) {
                call.cancel();
                return;
            }
        }
    }
}
var CALL_ID = 0;
var notClosedResponses = {};
var OkHttpResponse;
function createRequest(opts) {
    var client = getClient(false);
    var request = new okhttp3.Request.Builder();
    request.url(opts.url);
    if (opts.headers) {
        Object.keys(opts.headers).forEach(function (key) {
            return request.addHeader(key, opts.headers[key]);
        });
    }
    if (opts.cachePolicy) {
        var cacheControlBuilder = new okhttp3.CacheControl.Builder();
        switch (opts.cachePolicy) {
            case "noCache":
                cacheControlBuilder = cacheControlBuilder.noStore();
                break;
            case "onlyCache":
                cacheControlBuilder = cacheControlBuilder.onlyIfCached();
                break;
            case "ignoreCache":
                cacheControlBuilder = cacheControlBuilder.noCache();
                break;
        }
        request.cacheControl(cacheControlBuilder.build());
    }
    var methods = {
        GET: "get",
        HEAD: "head",
        DELETE: "delete",
        POST: "post",
        PUT: "put",
        PATCH: "patch",
    };
    var type;
    if (["GET", "HEAD"].indexOf(opts.method) !== -1 ||
        (opts.method === "DELETE" &&
            !types_1.isDefined(opts.body) &&
            !types_1.isDefined(opts.content))) {
        request[methods[opts.method]]();
    }
    else {
        type =
            opts.headers && opts.headers["Content-Type"]
                ? opts.headers["Content-Type"]
                : "application/json";
        var MEDIA_TYPE = okhttp3.MediaType.parse(type);
        var okHttpBody = void 0;
        if (type === "multipart/form-data") {
            var builder_1 = new okhttp3.MultipartBody.Builder();
            builder_1.setType(MEDIA_TYPE);
            opts.body.forEach(function (param) {
                if (param.fileName && param.contentType) {
                    var MEDIA_TYPE_1 = okhttp3.MediaType.parse(param.contentType);
                    builder_1.addFormDataPart(param.parameterName, param.fileName, okhttp3.RequestBody.create(MEDIA_TYPE_1, param.data));
                }
                else {
                    builder_1.addFormDataPart(param.parameterName, param.data);
                }
            });
            okHttpBody = builder_1.build();
        }
        else {
            var body = void 0;
            if (opts.body) {
                try {
                    body = JSON.stringify(opts.body);
                }
                catch (ignore) { }
            }
            else if (opts.content) {
                body = opts.content;
            }
            okHttpBody = okhttp3.RequestBody.create(okhttp3.MediaType.parse(type), body);
        }
        request[methods[opts.method]](okHttpBody);
    }
    var tag = "okhttp_request_" + CALL_ID++;
    var call = client.newCall(request.tag(tag).build());
    if (opts.useLegacy === false && opts.allowLargeResponse) {
        android.os.StrictMode.setThreadPolicy(android.os.StrictMode.ThreadPolicy.LAX);
    }
    return {
        nativeRequest: call,
        cancel: function () { return cancelRequest(tag, client); },
        run: function (resolve, reject) {
            call.enqueue(new okhttp3.Callback({
                onResponse: function (call, response) {
                    var responseBody = response.body();
                    var message = response.message();
                    var statusCode = response.code();
                    var getHeaders = function () {
                        var heads = response.headers();
                        var headers = {};
                        var i, len = heads.size();
                        for (i = 0; i < len; i++) {
                            var key = heads.name(i);
                            headers[key] = heads.value(i);
                        }
                        return headers;
                    };
                    if (opts.useLegacy) {
                        if (!OkHttpResponse) {
                            OkHttpResponse =
                                com.nativescript.https.OkHttpResponse;
                        }
                        var nResponse = new OkHttpResponse(responseBody);
                        if (opts.onProgress) {
                            nResponse.progressCallback = new OkHttpResponse.OkHttpResponseProgressCallback({
                                onProgress: opts.onProgress,
                            });
                        }
                        resolve({
                            response: response,
                            content: new HttpsResponse(nResponse, tag, opts.url),
                            statusCode: statusCode,
                            reason: message,
                            get headers() {
                                return getHeaders();
                            },
                        });
                    }
                    else {
                        resolve({
                            response: response,
                            content: responseBody.string(),
                            reason: message,
                            statusCode: statusCode,
                            get headers() {
                                return getHeaders();
                            },
                        });
                    }
                },
                onFailure: function (task, error) {
                    reject(error);
                },
            }));
        },
    };
}
exports.createRequest = createRequest;
function request(opts) {
    return new Promise(function (resolve, reject) {
        try {
            createRequest(opts).run(resolve, reject);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.request = request;
//# sourceMappingURL=https.android.js.map